@@ -44,7 +44,7 @@ class BaseDeDonnéesConfig(FichierConfig):
 class BaseDeDonnées:
     """Lien avec une base de données spécifique."""
 
-    def __init__(self, adresse: str, metadata: sqla.MetaData):
+    def __init__(self, adresse: str, metadata: sqla.MetaData, index_col='index'):
         """
         Lien avec la base de donnée se trouvant à adresse.
 
@@ -68,6 +68,8 @@ class BaseDeDonnées:
         # https://docs.sqlalchemy.org/en/14/core/schema.html
         self.metadata = metadata
 
+        self.index_col = index_col
+
     # Interface de sqlalchemy
 
     @property
@@ -120,7 +122,7 @@ class BaseDeDonnées:
         # Si une liste de colonnes est fournie, on vérifie qu'elles sont
         # toutes présentes dans le tableau.
         # On utilise aussi les objets Column du tableau
-        columns = [self.table(table).columns['index']] + list(
+        columns = [self.table(table).columns[self.index_col]] + list(
             filter(lambda x: x.name in columns, self.table(table).columns))
 
         requête = sqla.select(columns).select_from(self.table(table))
@@ -129,7 +131,7 @@ class BaseDeDonnées:
             requête = requête.where(clause)
 
         with self.begin() as con:
-            df = pd.read_sql(requête, con, index_col='index')
+            df = pd.read_sql(requête, con, index_col=self.index_col)
 
         return df
 
@@ -146,14 +148,12 @@ class BaseDeDonnées:
         :rtype: NoneType
 
         """
-        requête = self.table(table).update()
-
-        index = values.index.name
-        it = values.iterrows()
-        for i, rangée in it:
-            clause = self.table(table).columns[index] == i
-            r = requête.where(clause).values(**rangée)
-            self.execute(r)
+        with self.begin() as con:
+            for index, rangée in values.iterrows():
+                requête = self.table(table).update()\
+                                           .where(index == self.table(table).columns[self.index_col])\
+                                           .values(rangée)
+                con.execute(requête)
 
     def insert(self, table: str, values: pd.DataFrame):
         """
@@ -167,10 +167,8 @@ class BaseDeDonnées:
         :rtype: NoneType
 
         """
-        params = [({'index': i} | {c: (v if v else default(self.dtype(table, c))) for c, v in r.items()})
-                  for i, r in values.iterrows()]
-        requête = self.table(table).insert(params)
-        self.execute(requête)
+        with self.begin() as con:
+            values.to_sql(table, con, if_exists='append')
 
     def append(self, table: str, values: pd.DataFrame):
         """
@@ -184,18 +182,8 @@ class BaseDeDonnées:
         :rtype: NoneType
 
         """
-        # Réassigner les indices:
-        # On veut s'assurer qu'ils sont tous plus hauts
-        # que le dernier indice déjà dans le tableau.
-        indice_min = max(self.index(table), default=-1) + 1
-        nouvel_index = pd.Index(range(len(values.index)),
-                                name='index') + indice_min
-
-        # Faire une copie des valeurs avec le bon index.
-        values = values.copy()
-        values.index = nouvel_index
-
-        self.insert(table, values)
+        with self.begin() as con:
+            values.to_sql(table, con, if_exists='append')
 
     def delete(self, table: str, values: pd.DataFrame):
         """
@@ -216,8 +204,8 @@ class BaseDeDonnées:
             index = values.index.name
             idx = values.index
         else:
-            index = 'index'
-            idx = pd.Index([values], name='index')
+            index = self.index_col
+            idx = pd.Index([values], name=index)
         for i in idx:
             clause = self.table(table).columns[index] == 1
             r = requête.where(clause)
@@ -359,7 +347,7 @@ class BaseDeDonnées:
 
         """
         res = pd.Index(c.name for c in self.table(
-            table).columns if c.name != 'index')
+            table).columns if c.name != self.index_col)
 
         return res
 
@@ -374,11 +362,11 @@ class BaseDeDonnées:
 
         """
         requête = sqla.select([self.table(
-            table).columns['index']]).select_from(self.table(table))
+            table).columns[self.index_col]]).select_from(self.table(table))
 
         with self.begin() as con:
             résultat = con.execute(requête)
-            res = pd.Index(r['index'] for r in résultat)
+            res = pd.Index(r[self.index_col] for r in résultat)
             return res
 
     def loc(self,
@@ -470,7 +458,7 @@ class BaseDeDonnées:
         elif isinstance(type_fichier, str):
             type_fichier = TYPES_FICHIERS[type_fichier]
 
-        df = type_fichier(chemin, index_col='index')
+        df = type_fichier(chemin, index_col=self.index_col)
 
         self.màj(table, df)
 
@@ -478,7 +466,7 @@ class BaseDeDonnées:
 class BaseTableau:
     """Encapsulation de la classe BaseDeDonnées."""
 
-    def __init__(self, db: BaseDeDonnées, table: str):
+    def __init__(self, db: BaseDeDonnées, table: str, index_col: str = 'index'):
         """
         Encapsule de la classe BaseDeDonnées.
 
@@ -493,7 +481,15 @@ class BaseTableau:
 
         """
         self.table: str = table
-        self.db: BaseDeDonnées = db
+        self.index_col = index_col
+
+        if isinstance(db, str):
+            self.db: BaseDeDonnées = BaseDeDonnées(
+                db, sqla.MetaData(), index_col)
+            moteur = self.db.create_engine()
+            self.db.metadata.reflect(moteur)
+        else:
+            self.db: BaseDeDonnées = db
 
     def __getattr__(self, attr: str) -> Any:
         """
@@ -507,26 +503,33 @@ class BaseTableau:
         :raises AttributeError: Si l'attribut ne peut pas être trouvé.
 
         """
-        if hasattr(BaseDeDonnées, attr):
-            obj = getattr(self.db, attr)
+        résultat = None
+        try:
+            # Pour utiliser des colonnes d'index différentes, avec une même base de données.
+            self.db.index_col, vieux_index_col = self.index_col, self.db.index_col
+            if hasattr(BaseDeDonnées, attr):
+                obj = getattr(self.db, attr)
+                if isinstance(obj, Callable):
+                    sig = signature(obj)
 
-            if isinstance(obj, Callable):
-                sig = signature(obj)
-
-                if len(sig.parameters) == 1 and 'table' in sig.parameters:
-                    return partial(obj, self.table)()
-                elif 'table' in sig.parameters:
-                    return partial(obj, self.table)
+                    if len(sig.parameters) == 1 and 'table' in sig.parameters:
+                        résultat = partial(obj, self.table)()
+                    elif 'table' in sig.parameters:
+                        résultat = partial(obj, self.table)
+                    else:
+                        résultat = obj
                 else:
-                    return obj
+                    résultat = obj
+            elif hasattr(pd.DataFrame, attr):
+                résultat = getattr(self.df, attr)
             else:
-                return obj
-        elif hasattr(pd.DataFrame, attr):
-            return getattr(self.df, attr)
-        else:
-            msg = f'{self!r} de type {type(self)} n\'a pas d\'attribut {attr}\
-, ni (self.__db: BaseDeDonnées, self.df: pandas.DataFrame).'
-            raise AttributeError(msg)
+                msg = f'{self!r} de type {type(self)} n\'a pas d\'attribut {attr}\
+    , ni (self.__db: BaseDeDonnées, self.df: pandas.DataFrame).'
+                raise AttributeError(msg)
+        finally:
+            self.db.index_col = vieux_index_col
+
+        return résultat
 
     @ property
     def df(self) -> pd.DataFrame:
